package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.model.MaterialType
import dev.fyloz.colorrecipesexplorer.model.MaterialTypeSaveDto
import dev.fyloz.colorrecipesexplorer.model.MaterialTypeUpdateDto
import dev.fyloz.colorrecipesexplorer.service.MaterialTypeService
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

private const val MATERIAL_TYPE_CONTROLLER_PATH = "api/materialtype"

@RestController
@RequestMapping(MATERIAL_TYPE_CONTROLLER_PATH)
@PreAuthorizeViewCatalog
class MaterialTypeController(private val materialTypeService: MaterialTypeService) {
    @GetMapping
    fun getAll() =
        ok(materialTypeService.getAll())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(materialTypeService.getById(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun save(@Valid @RequestBody materialType: MaterialTypeSaveDto) =
        created<MaterialType>(MATERIAL_TYPE_CONTROLLER_PATH) {
            materialTypeService.save(materialType)
        }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_MATERIAL_TYPES')")
    fun update(@Valid @RequestBody materialType: MaterialTypeUpdateDto) =
        noContent {
            materialTypeService.update(materialType)
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('REMOVE_MATERIAL_TYPEs')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            materialTypeService.deleteById(id)
        }
}

